library(ggplot2)
library(ncdf4)
library(trend)
library(loadeR)
library(transformeR)
library(visualizeR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Working directory
dir <- "/esarchive/scratch/molmo/CLIMCAT_MJ/CMIP6/"
OUTPUTS <- "/esarchive/scratch/molmo/CLIMCAT_MJ/OUTPUTS/"
setwd(dir)

invariable <- loadGridData(dataset = "/esarchive/scratch/molmo/mask_1_Med_bil.nc",
                           var="precip") %>% binaryGrid(condition = "GE",threshold=0)

#-------Annual indices---------#
#Useful information
indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")
files <- intersect(list.files(pattern="2100.nc",recursive = T),list.files(pattern="yr",recursive = T))
files <- intersect(files,list.files(pattern="ssp585",recursive=T))

ssn <- "Annual"
seasons <- list(c(1:12))
Model <- NA

series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive=T))
  series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-ssp585_r1i1p1f1_1950-2100.nc") 
      Model[m] <- substr(files_subset[m],start=ini+32,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),years=1950:2100)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
      for(y in 1:dim(data$Data)[1]){
        data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      }
      
      #Estimating anomalies wrt 1981-2010 reference period
      data <- scaleGrid(data,base = subsetGrid(data,years=1981:2010),type = "center")
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
    }
  }
}
save(series.indices,file="series.indices.yr.CMIP6.ssp585.Rdata")

#--------Seasonal indices--------------------#
files <- intersect(list.files(pattern="210012.nc",recursive = T),list.files(pattern="",recursive = T))
files <- intersect(files,list.files(pattern="ssp585",recursive=T))

seasons <- list(c(12,1,2),c(3,4,5),c(6,7,8),c(9,10,11)) #DJF #MAM #JJA #SON #ANNUAL
ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")
indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")
Model <- NA

series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive = T))

    series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-ssp585_r1i1p1f1_195001-210012.nc") 
      Model[m] <- substr(files_subset[m],start=ini+33,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),years=1950:2100)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
      for(y in 1:dim(data$Data)[1]){
        data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      }
      
      #Estimating anomalies wrt 1981-2010 reference period
      data <- scaleGrid(data,base = subsetGrid(data,years=1981:2010),type = "center")
      
      data <- aggregateGrid(data,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      #trend.indices[[i]][[m]][[s]] <- datita #tal vez acá va el data.frame
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
    }
  }
}
save(series.indices,file="series.indices.mon.CMIP6.ssp585.Rdata")




######.   VIEJO   .######
#--------Ploteo de heatmap-------#

#Annual
load("series.indices.yr.CMIP6.historical.Rdata")
indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(series.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",series.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- toupper(test3$Index)
test3$Index <- factor(test3$Index,levels=toupper(c("cdd","cwd","prcptot","r20mm","csdi","wsdi","tr")))
#trend <- aggregate(Tau~Model+Index,test3,FUN="mean",na.rm=T)
#trend$Significance <- sign$Significance
#trend$Model[which(trend$Model=="")] <- "ERA5"

png(filename = paste(OUTPUTS,"Heatmap_Tau_Annual_Trends_1950-2014",".png",sep=""),res=300,width=3000,height=2000)
print(
  ggplot(data=test3,aes(x=Index,y=Model,fill=Tau)) + theme_bw() + geom_tile() + coord_cartesian(expand=F)+
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdYlBu")),limits=c(-0.6,0.6),
                          aesthetics = "fill",na.value = "white") + ggtitle("Annual")+ facet_wrap(~Region,nrow = 1)+
    geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()

#---------------------------#
#Seasonal
load("series.indices.mon.CMIP6.historical.Rdata")
indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(series.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",series.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Index <- toupper(test3$Index)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- factor(test3$Index,levels=toupper(c("tnn","tnx","txn","txx","tn10p","tn90p","tx10p","tx90p",
                                                   "rx1day","rx5day")))

test3$Season <- factor(test3$Season,levels=c("Summer","Autumn","Winter","Spring"))

test4 <- test3

test3 <- subset(test4,Region=="Catalonia")
png(filename = paste(OUTPUTS,"Heatmap_Tau_Seasonal_Catalonia_Trends_1950-2014",".png",sep=""),res=300,width=4000,height=3000)
print(
  ggplot(data=test3,aes(x=Index,y=Model,fill=Tau)) + theme_bw() + geom_tile() + coord_cartesian(expand=F)+
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdYlBu")),limits=c(-0.65,0.65),
                          aesthetics = "fill",na.value = "white") + ggtitle("Seasonal")+ facet_wrap(~Season)+
    geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()

test3 <- subset(test4,Region=="EuroMediterranean")
png(filename = paste(OUTPUTS,"Heatmap_Tau_Seasonal_EuroMediterranean_Trends_1950-2014",".png",sep=""),res=300,width=4000,height=3000)
print(
  ggplot(data=test3,aes(x=Index,y=Model,fill=Tau)) + theme_bw() + geom_tile() + coord_cartesian(expand=F)+
    scale_color_gradientn("Tau",colours = rev(brewer.pal(n=11,name="RdYlBu")),limits=c(-0.65,0.65),
                          aesthetics = "fill",na.value = "white") + ggtitle("Seasonal")+ facet_wrap(~Season)+
    geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()

#################################################################################
#-------------------------#
#Differentiated model ensembles based on historical performance

test3$Performance <- NA
test3$Performance[which(test3$Model=="ERA5")] <- "ERA5"
test3$Performance[which(test3$Model%in%c("EC-Earth3","EC-Earth3-CC","EC-Earth3-Veg","EC-Earth3-AerChem","EC-Earth3-Veg-LR","MIROC6","MRI-ESM2-0","TaiESM1"))] <- "Good"
test3$Performance[which(test3$Model%in%c("ACCESS-CM2","ACCESS-ESM1-5","AWI-ESM-1-1-LR","IPSL-CM6A-LR","MPI-ESM1-2-HR","MPI-ESM1-2-LR","NorESM2-MM"))] <- "Normal"
test3$Performance[which(test3$Model%in%c("CanESM5","CMCC-ESM2","FGOALS-g3","INM-CM4-8","INM-CM5-0","GFDL-ESM4","KACE-1-0-G","NESM3","NorESM2-LM"))] <- "Worst"

png(filename = paste(OUTPUTS,"Boxplot_Tau_Seasonal","_P.png",sep=""),res=300,width=3000,height=2000)
print(
  ggplot(data=test3,aes(x=Index,y=Tau,colour=Performance)) + theme_bw() + geom_boxplot() +# coord_cartesian(expand=F)+
    ggtitle("Seasonal")+ facet_grid(Season~Region)#+
  #geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()

png(filename = paste(OUTPUTS,"Boxplot_Tau_Annual","_P.png",sep=""),res=300,width=3000,height=2000)
print(
  ggplot(data=test3,aes(x=Index,y=Tau,colour=Performance)) + theme_bw() + geom_boxplot() +# coord_cartesian(expand=F)+
    ggtitle("Annual")+ facet_grid(Season~Region)#+
  #geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()




#######
test3$Performance <- NA
test3$Performance[which(test3$Model=="ERA5")] <- "ERA5"
test3$Performance[which(test3$Model%in%c("EC-Earth3","GFDL-CM4","EC-Earth3-CC","EC-Earth3-Veg","MIROC6","TaiESM1","GFDL-ESM4"))] <- "Good"
test3$Performance[which(test3$Model%in%c("EC-Earth3-AerChem","EC-Earth3-Veg-LR","CMCC-ESM2","AWI-ESM-1-1-LR","IPSL-CM6A-LR","MPI-ESM1-2-HR","MPI-ESM1-2-LR","NorESM2-MM"))] <- "Normal"
test3$Performance[which(test3$Model%in%c("MRI-ESM2-0","CanESM5","ACCESS-CM2","ACCESS-ESM1-5","FGOALS-g3","INM-CM4-8","INM-CM5-0","KACE-1-0-G","NESM3","NorESM2-LM"))] <- "Worst"

png(filename = paste(OUTPUTS,"Boxplot_Tau_Seasonal","_2.png",sep=""),res=300,width=3000,height=2000)
print(
  ggplot(data=test3,aes(x=Index,y=Tau,colour=Performance)) + theme_bw() + geom_boxplot() +# coord_cartesian(expand=F)+
    ggtitle("Seasonal")+ facet_grid(Season~Region)#+
  #geom_text(data=subset(test3,pv<=0.05),aes(x=Index,y=Model,label="*",fontface="bold"))
)
dev.off()

######################################################
#--------------------Ensemble map----------------###
agreement <- function(x){
  sig <- sign(x)
  pos <- length(which(sig>0))/length(sig)*100
  neg <- length(which(sig<0))/length(sig)*100
  value <- max(pos,neg)
  if(value<50){ag <- 0}
  if(value>=50 & value<80){ag <- 1}
  if(value>=80){ag <- 2}
  return(ag)
}

#Annual
load("trend.indices.yr.CMIP6.historical.Rdata")

indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(trend.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",trend.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- toupper(test3$Index)
test3$Index <- factor(test3$Index,levels=toupper(c("cdd","cwd","prcptot","r20mm","csdi","wsdi","tr")))

ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)

ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
ensemble$Agreement <- ensemble2$Tau
ensemble$Tau[which(ensemble$Index=="CDD")] <- ensemble$Tau[which(ensemble$Index=="CDD")]*-1

ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.4
ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.4

p <- ggplot(data=subset(ensemble,Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("CMIP6 ensemble: annual indices")+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("cdd","cwd","r20mm","prcptot")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

g <- ggplot(data=subset(ensemble,Index%in%toupper(c("csdi","wsdi","tr"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("CMIP6 ensemble: annual indices")+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("csdi","wsdi","tr"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("csdi","wsdi","tr")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

###ERA5---#
ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
ensemble$Tau[which(ensemble$Index=="CDD")] <- ensemble$Tau[which(ensemble$Index=="CDD")]*-1

ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.4
ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.4

h <- ggplot(data=subset(ensemble,Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile() + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

i <- ggplot(data=subset(ensemble,Index%in%toupper(c("csdi","wsdi","tr"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

png(filename = paste(OUTPUTS,"AnnualIndices_CMIP6_Trends_1950-2014_Map",".png",sep=""),res=300,width=4000,height=2000)
print(
  grid.arrange(p,h,g,i,nrow=2)
)
dev.off() 

#---------------------------------------------------------#
#Seasonal
load("trend.indices.mon.CMIP6.historical.Rdata")

indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")
ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(trend.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",trend.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Index <- toupper(test3$Index)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- factor(test3$Index,levels=toupper(c("tnn","tnx","txn","txx","tn10p","tn90p","tx10p","tx90p",
                                                   "rx1day","rx5day")))

test3$Season <- factor(test3$Season,levels=c("Summer","Autumn","Winter","Spring"))

tes.t <- test3

for(j in 1:length(ssn)){
  test3 <- subset(tes.t,Season==ssn[j])
  
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  
  ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
  ensemble$Agreement <- ensemble2$Tau
  
  ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.4
  ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.4
  
  p <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g1 <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g2 <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  pcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g1cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g2cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  ###ERA5---#
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  
  ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.4
  ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.4
  
  h <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile() + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  i <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1 <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  hcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile() + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  icat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  png(filename = paste(OUTPUTS,"SeasonalIndices_TX_TN_CMIP6_Trends_1950-2014_Map_",ssn[j],".png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(#p,h,
      g1,i,
      g2,j1,nrow=2)
  )
  dev.off() 
  
  png(filename = paste(OUTPUTS,"SeasonalIndices_PR_CMIP6_Trends_1950-2014_Map_",ssn[j],".png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(p,h,p,h,
                 nrow=2)
  )
  dev.off() 
  
  #---Catalonia---#
  hcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile() + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  icat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,fill=Tau)) + geom_tile()  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  png(filename = paste(OUTPUTS,"SeasonalIndices_TX_TN_CMIP6_Trends_1950-2014_Map_Catalonia_",ssn[j],".png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(#p,h,
      g1cat,icat,
      g2cat,j1cat,nrow=2)
  )
  dev.off() 
  
  png(filename = paste(OUTPUTS,"SeasonalIndices_PR_CMIP6_Trends_1950-2014_Map_Catalonia_",ssn[j],".png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(pcat,hcat,pcat,hcat,
                 nrow=2)
  )
  dev.off() 
}
